@@ -13,24 +13,36 @@ use Fey::Validate
         POS_INTEGER_TYPE POS_OR_ZERO_INTEGER_TYPE
         TABLE_TYPE );
 
+use Fey::Column::Alias;
+use Fey::Literal;
+use Fey::Table;
+use Fey::Table::Alias;
+
 use Moose::Policy 'MooseX::Policy::SemiAffordanceAccessor';
 use MooseX::StrictConstructor;
-use Moose::Util::TypeConstraints qw( subtype as where coerce from via );
+use Moose::Util::TypeConstraints qw( subtype as where coerce from via class_type find_type_constraint );
 
 with 'Fey::Role::ColumnLike';
 
+has 'id' =>
+    ( is         => 'ro',
+      lazy_build => 1,
+      init_arg   => undef,
+      clearer    => '_clear_id',
+    );
+
 has 'name' =>
     ( is       => 'ro',
       isa      => 'Str',
       required => 1,
     );
 
-subtype 'GenericTypeName'
+subtype 'Fey.Type.GenericTypeName'
     => as 'Str'
     => where { /^(?:text|blob|integer|float|date|datetime|time|boolean|other)$/xism };
 has 'generic_type' =>
     ( is         => 'ro',
-      isa        => 'GenericTypeName',
+      isa        => 'Fey.Type.GenericTypeName',
       lazy_build => 1,
     );
 
@@ -40,22 +52,22 @@ has type =>
       required => 1,
     );
 
-subtype 'PosInteger'
+subtype 'Fey.Type.PosInteger'
     => as 'Int'
     => where { $_ > 0 };
 has length =>
     ( is       => 'ro',
-      isa      => 'PosInteger',
+      isa      => 'Fey.Type.PosInteger',
       required => 0
     );
 
-subtype 'PosOrZeroInteger'
+subtype 'Fey.Type.PosOrZeroInteger'
     => as 'Int'
     => where { $_ >= 0 };
 # How to say that precision requires length as well?
 has precision =>
     ( is       => 'ro',
-      isa      => 'PosOrZeroInteger',
+      isa      => 'Fey.Type.PosOrZeroInteger',
       required => 0
     );
 
@@ -71,9 +83,9 @@ has is_nullable =>
       default => 0,
     );
 
-subtype 'DefaultValue'
+subtype 'Fey.Type.DefaultValue'
     => as 'Fey::Literal';
-coerce 'DefaultValue'
+coerce 'Fey.Type.DefaultValue'
     => from 'Undef'
     => via { Fey::Literal::Null->new() }
     => from 'Value'
@@ -81,21 +93,29 @@ coerce 'DefaultValue'
 
 has default =>
     ( is     => 'ro',
-      isa    => 'DefaultValue',
+      isa    => 'Fey.Type.DefaultValue',
       coerce => 1,
     );
 
-has 'table' =>
-    ( is       => 'rw',
-      isa      => 'Undef | Fey::Table | Fey::Table::Alias',
-      weak_ref => 1,
-      writer   => '_set_table',
-    );
+{
+    for my $class ( qw( Fey::Table Fey::Table::Alias ) )
+    {
+        class_type($class)
+            unless find_type_constraint($class);
+    }
+
+    has 'table' =>
+        ( is       => 'rw',
+          isa      =>  'Fey::Table | Fey::Table::Alias',
+          weak_ref => 1,
+          writer   => '_set_table',
+          clearer  => '_clear_table',
+        );
+}
+
+after '_set_table', '_clear_table' =>
+    sub { $_[0]->_clear_id() };
 
-use Fey::Column::Alias;
-use Fey::Literal;
-use Fey::Table;
-use Fey::Table::Alias;
 
 
 {
@@ -161,20 +181,22 @@ sub sql_with_alias { goto &sql }
 
 sub sql_or_alias { goto &sql }
 
-sub id
+sub _build_id
 {
     my $self = shift;
 
     my $table = $self->table();
 
     object_state_error
-        'The id() method cannot be called on a column object which has no table.'
+        'The id attribute cannot be determined for a column object which has no table.'
             unless $table;
 
     return $table->id() . q{.} . $self->name();
 }
 
 no Moose;
+no Moose::Util::TypeConstraints;
+
 __PACKAGE__->meta()->make_immutable();
 
 1;
